@@ -36,6 +36,7 @@ pub use rlp;
 use crate::types::{Address, Bytes, U256};
 use k256::{ecdsa::SigningKey, EncodedPoint as K256PublicKey};
 use std::convert::TryInto;
+use std::ops::Neg;
 use thiserror::Error;
 
 #[derive(Debug, Error)]
@@ -47,6 +48,19 @@ pub enum FormatBytes32StringError {
 /// 1 Ether = 1e18 Wei == 0x0de0b6b3a7640000 Wei
 pub const WEI_IN_ETHER: U256 = U256([0x0de0b6b3a7640000, 0x0, 0x0, 0x0]);
 
+/// The number of blocks from the past for which the fee rewards are fetched for fee estimation.
+pub const EIP1559_FEE_ESTIMATION_PAST_BLOCKS: u64 = 10;
+/// The default percentile of gas premiums that are fetched for fee estimation.
+pub const EIP1559_FEE_ESTIMATION_REWARD_PERCENTILE: f64 = 5.0;
+/// The default max priority fee per gas, used in case the base fee is within a threshold.
+pub const EIP1559_FEE_ESTIMATION_DEFAULT_PRIORITY_FEE: u64 = 3_000_000_000;
+/// The threshold for base fee below which we use the default priority fee, and beyond which we
+/// estimate an appropriate value for priority fee.
+pub const EIP1559_FEE_ESTIMATION_PRIORITY_FEE_TRIGGER: u64 = 100_000_000_000;
+/// The threshold max change/difference (in %) at which we will ignore the fee history values
+/// under it.
+pub const EIP1559_FEE_ESTIMATION_THRESHOLD_MAX_CHANGE: i64 = 200;
+
 /// Format the output for the user which prefer to see values
 /// in ether (instead of wei)
 ///
@@ -65,7 +79,7 @@ pub fn format_units<T: Into<U256>, K: Into<Units>>(amount: T, units: K) -> U256
 /// Converts the input to a U256 and converts from Ether to Wei.
 ///
 /// ```
-/// use ethers::{types::U256, utils::{parse_ether, WEI_IN_ETHER}};
+/// use ethers_core::{types::U256, utils::{parse_ether, WEI_IN_ETHER}};
 ///
 /// let eth = U256::from(WEI_IN_ETHER);
 /// assert_eq!(eth, parse_ether(1u8).unwrap());
@@ -187,10 +201,97 @@ pub fn parse_bytes32_string(bytes: &[u8; 32]) -> Result<&str, std::str::Utf8Erro
     std::str::from_utf8(&bytes[..length])
 }
 
+/// The default EIP-1559 fee estimator which is based on the work by [MyCrypto](https://github.com/MyCryptoHQ/MyCrypto/blob/master/src/services/ApiService/Gas/eip1559.ts)
+pub fn eip1559_default_estimator(base_fee_per_gas: U256, rewards: Vec<Vec<U256>>) -> (U256, U256) {
+    let max_priority_fee_per_gas =
+        if base_fee_per_gas < U256::from(EIP1559_FEE_ESTIMATION_PRIORITY_FEE_TRIGGER) {
+            U256::from(EIP1559_FEE_ESTIMATION_DEFAULT_PRIORITY_FEE)
+        } else {
+            std::cmp::max(
+                estimate_priority_fee(rewards),
+                U256::from(EIP1559_FEE_ESTIMATION_DEFAULT_PRIORITY_FEE),
+            )
+        };
+    let potential_max_fee = base_fee_surged(base_fee_per_gas);
+    let max_fee_per_gas = if max_priority_fee_per_gas > potential_max_fee {
+        max_priority_fee_per_gas + potential_max_fee
+    } else {
+        potential_max_fee
+    };
+    (max_fee_per_gas, max_priority_fee_per_gas)
+}
+
+fn estimate_priority_fee(rewards: Vec<Vec<U256>>) -> U256 {
+    let mut rewards: Vec<U256> = rewards
+        .iter()
+        .map(|r| r[0])
+        .filter(|r| *r > U256::zero())
+        .collect();
+    if rewards.is_empty() {
+        return U256::zero();
+    }
+    if rewards.len() == 1 {
+        return rewards[0];
+    }
+    // Sort the rewards as we will eventually take the median.
+    rewards.sort();
+
+    // A copy of the same vector is created for convenience to calculate percentage change
+    // between subsequent fee values.
+    let mut rewards_copy = rewards.clone();
+    rewards_copy.rotate_left(1);
+
+    let mut percentage_change: Vec<i64> = rewards
+        .iter()
+        .zip(rewards_copy.iter())
+        .map(|(a, b)| {
+            if b > a {
+                ((b - a).low_u32() as i64 * 100) / (a.low_u32() as i64)
+            } else {
+                (((a - b).low_u32() as i64 * 100) / (a.low_u32() as i64)).neg()
+            }
+        })
+        .collect();
+    percentage_change.pop();
+
+    // Fetch the max of the percentage change, and that element's index.
+    let max_change = percentage_change.iter().max().unwrap();
+    let max_change_index = percentage_change
+        .iter()
+        .position(|&c| c == *max_change)
+        .unwrap();
+
+    // If we encountered a big change in fees at a certain position, then consider only
+    // the values >= it.
+    let values = if *max_change >= EIP1559_FEE_ESTIMATION_THRESHOLD_MAX_CHANGE
+        && (max_change_index >= (rewards.len() / 2))
+    {
+        rewards[max_change_index..].to_vec()
+    } else {
+        rewards
+    };
+
+    // Return the median.
+    values[values.len() / 2]
+}
+
+fn base_fee_surged(base_fee_per_gas: U256) -> U256 {
+    if base_fee_per_gas <= U256::from(40_000_000_000u64) {
+        base_fee_per_gas * 2
+    } else if base_fee_per_gas <= U256::from(100_000_000_000u64) {
+        base_fee_per_gas * 16 / 10
+    } else if base_fee_per_gas <= U256::from(200_000_000_000u64) {
+        base_fee_per_gas * 14 / 10
+    } else {
+        base_fee_per_gas * 12 / 10
+    }
+}
+
 /// A bit of hack to find an unused TCP port.
 ///
 /// Does not guarantee that the given port is unused after the function exists, just that it was
 /// unused before the function started (i.e., it does not reserve a port).
+#[cfg(not(target_arch = "wasm32"))]
 pub(crate) fn unused_port() -> u16 {
     let listener = std::net::TcpListener::bind("127.0.0.1:0")
         .expect("Failed to create TCP listener to find unused port");
@@ -451,4 +552,33 @@ mod tests {
             FormatBytes32StringError::TextTooLong
         ));
     }
+
+    #[test]
+    fn test_eip1559_default_estimator() {
+        // If the base fee is below the triggering base fee, we should get the default priority fee
+        // with the base fee surged.
+        let base_fee_per_gas = U256::from(EIP1559_FEE_ESTIMATION_PRIORITY_FEE_TRIGGER) - 1;
+        let rewards: Vec<Vec<U256>> = vec![vec![]];
+        let (base_fee, priority_fee) = eip1559_default_estimator(base_fee_per_gas, rewards);
+        assert_eq!(
+            priority_fee,
+            U256::from(EIP1559_FEE_ESTIMATION_DEFAULT_PRIORITY_FEE)
+        );
+        assert_eq!(base_fee, base_fee_surged(base_fee_per_gas));
+
+        // If the base fee is above the triggering base fee, we calculate the priority fee using
+        // the fee history (rewards).
+        let base_fee_per_gas = U256::from(EIP1559_FEE_ESTIMATION_PRIORITY_FEE_TRIGGER) + 1;
+        let rewards: Vec<Vec<U256>> = vec![
+            vec![100_000_000_000u64.into()],
+            vec![105_000_000_000u64.into()],
+            vec![102_000_000_000u64.into()],
+        ]; // say, last 3 blocks
+        let (base_fee, priority_fee) = eip1559_default_estimator(base_fee_per_gas, rewards.clone());
+        assert_eq!(base_fee, base_fee_surged(base_fee_per_gas));
+        assert_eq!(priority_fee, estimate_priority_fee(rewards.clone()));
+
+        // The median should be taken because none of the changes are big enough to ignore values.
+        assert_eq!(estimate_priority_fee(rewards), 102_000_000_000u64.into());
+    }
 }
